
# q-expansion of newform 476.2.a.c, downloaded from the LMFDB on 10 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 476
weight = 2
poly_data = [-3, -1, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0, 0], [0, -1], [1, -1], [-1, 0], [4, 0], [0, -2], [-1, 0], [4, 2], [4, 0], [0, 4], [5, 1], [-4, 6], [0, -5], [-2, -1], [2, -2], [3, -1], [-8, 0], [4, 5], [5, -1], [-8, 2], [-4, 5], [2, -2], [-6, 4], [-6, 6], [-11, -1], [-8, -2], [14, 0], [0, -2], [14, -4], [-14, 2], [3, 5], [0, -4], [2, 7], [-3, 5], [-2, -1], [14, -1], [2, 0], [6, -2], [-4, 7], [0, 9], [-2, 5], [-6, 0], [9, -9], [8, -10], [-2, -2], [12, -1], [20, -6], [0, 6], [9, -3], [8, -4], [22, 2], [2, 7], [-15, -5], [-16, 6], [14, -8], [0, -8], [22, -4], [8, -8], [20, -8], [-3, -7], [-5, 11], [-2, -4], [4, 0], [3, -9], [-3, 3], [-6, -12], [-17, 9], [4, -8], [-10, 16], [0, 8], [-16, 0], [-13, 5], [0, 3], [10, -11], [28, -8], [-20, 8], [-9, -9], [-19, 3], [-2, 2], [8, -6], [3, -5], [18, -3], [8, -16], [-12, 2], [28, -3], [-24, -4], [18, -10], [-7, -3], [28, -2], [14, -9], [-12, -2], [24, -3], [-28, 0], [-11, -5], [-20, -4], [-15, -7], [2, 2], [-12, 5], [-8, 20], [4, 0], [2, 0], [-30, 4], [4, 0], [6, 7], [-4, -18], [-22, 6], [-16, 2], [36, -6], [6, 9], [-10, 24], [-25, 11], [-1, -9], [-20, 12], [28, 4], [-7, 7], [16, 12], [24, -7], [-8, -12], [22, 8], [5, 7], [18, -10], [16, -10], [-6, 0], [18, -2], [28, -13], [-30, 8], [28, -14], [12, -1], [4, -8], [-16, 14], [2, -11], [-30, -2], [8, -2], [-10, 1], [-18, -14], [10, -20], [-6, 10], [32, 8], [2, 20], [-6, -6], [-16, 5], [24, -14], [-34, -2], [36, 4], [16, -14], [0, 8], [-10, 0], [7, -23], [-50, -2], [-25, 21], [-6, 2], [3, -7], [19, -3], [-11, 29], [18, 4], [-2, -6], [-22, 21], [43, -3], [-28, -4], [-36, 1], [-12, 16], [27, -21], [2, -5], [12, -4], [-11, 1], [-19, 29], [-2, -4], [-48, 1]]
